import {
  world,
  system
} from "@minecraft/server";

import { CHAT_COOLDOWN } from "./config.js";

export const COOLDOWNS = {};

export default class Cooldown {
    /**
     * Generates a cooldown for a player
     * @param {Player.nameTag} player player whos the cooldown is for
     */
    constructor(player) {
      this.player = player;
      this.time = Date.now() + CHAT_COOLDOWN * 1000;
      const time = this.time;
      let systExpire = false
      
      const expire = () => {
        delete COOLDOWNS[this.player];
        for (const key in this.events) {
          world.events[key].unsubscribe(this.events[key]);
        }
      }

      system.run(function tick(data) {
        if (time <= Date.now() && systExpire == false){
          systExpire = true
          expire()
        } 
        system.run(tick)
      })
  
      COOLDOWNS[this.player] = this;
      this.events = {
        playerLeave: world.events.playerLeave.subscribe(({ playerName }) => {
          if (playerName == this.player) expire();
        }),
      };
    }
  
    /**
     * Caculates the time left of the cooldown
     * @returns {number}
     */
    get TIME_LEFT() {
      return Math.ceil((this.time - Date.now()) / 1000);
    }
  
    /**
     * Sends a chat reply
     * @returns {void}
     */
    reply(player) {
      return player.tell(
        `§7[§r§3Chat§bColors§7]§r§c Don't spam the chat §c!`
      );
    }
  }