import { MolangVariableMap } from "@minecraft/server";
import { Shape } from "./base_shape.js";
import { Vector } from "./../../library/Minecraft.js";
export class SphereShape extends Shape {
    constructor(radiusX, radiusY, radiusZ, domeDirection) {
        super();
        this.radii = [0, 0, 0];
        this.customHollow = true;
        this.radii[0] = radiusX;
        this.radii[1] = radiusY ?? this.radii[0];
        this.radii[2] = radiusZ ?? this.radii[1];
        this.domeDirection = domeDirection;
    }
    getRegion(loc) {
        if (this.domeDirection?.x === 1) {
            return [loc.offset(0, -this.radii[1], -this.radii[2]), loc.offset(this.radii[0], this.radii[1], this.radii[2])];
        }
        else if (this.domeDirection?.x === -1) {
            return [loc.offset(-this.radii[0], -this.radii[1], -this.radii[2]), loc.offset(0, this.radii[1], this.radii[2])];
        }
        else if (this.domeDirection?.y === 1) {
            return [loc.offset(-this.radii[0], 0, -this.radii[2]), loc.offset(this.radii[0], this.radii[1], this.radii[2])];
        }
        else if (this.domeDirection?.y === -1) {
            return [loc.offset(-this.radii[0], -this.radii[1], -this.radii[2]), loc.offset(this.radii[0], 0, this.radii[2])];
        }
        else if (this.domeDirection?.z === 1) {
            return [loc.offset(-this.radii[0], -this.radii[1], 0), loc.offset(this.radii[0], this.radii[1], this.radii[2])];
        }
        else if (this.domeDirection?.z === -1) {
            return [loc.offset(-this.radii[0], -this.radii[1], -this.radii[2]), loc.offset(this.radii[0], this.radii[1], 0)];
        }
        else {
            return [loc.offset(-this.radii[0], -this.radii[1], -this.radii[2]), loc.offset(this.radii[0], this.radii[1], this.radii[2])];
        }
    }
    getYRange() {
        throw new Error("getYRange not implemented!");
    }
    draw(loc, player, global) {
        const dimension = player.dimension;
        try {
            loc = loc.add(0.5);
            const spawnAt = Vector.add(player.getHeadLocation(), Vector.from(player.getViewDirection()).mul(20));
            spawnAt.y = Math.min(Math.max(spawnAt.y, dimension.heightRange.min), dimension.heightRange.max);
            const molangVars = new MolangVariableMap();
            molangVars.setVector3("offset", Vector.sub(loc, spawnAt));
            if (this.domeDirection) {
                // Dome
                molangVars.setVector3("direction", this.domeDirection);
                molangVars.setFloat("radius", this.radii[0] + 0.5);
                (global ? dimension : player).spawnParticle("wedit:wireframe_dome", spawnAt, molangVars);
            }
            else if (!(this.radii[0] === this.radii[1] && this.radii[1] === this.radii[2])) {
                // Ellipsoid
                molangVars.setVector3("radii", { x: this.radii[0] + 0.5, y: this.radii[1] + 0.5, z: this.radii[2] + 0.5 });
                (global ? dimension : player).spawnParticle("wedit:wireframe_ellipsoid", spawnAt, molangVars);
            }
            else {
                // Sphere
                molangVars.setFloat("radius", this.radii[0] + 0.5);
                (global ? dimension : player).spawnParticle("wedit:wireframe_sphere", spawnAt, molangVars);
            }
        }
        catch {
            /* pass */
        }
    }
    prepGeneration(genVars, options) {
        genVars.isHollow = options?.hollow ?? false;
        genVars.radiiOff = this.radii.map((v) => v + 0.5);
        genVars.thickness = options?.hollowThickness ?? 1;
    }
    inShape(relLoc, genVars) {
        if (genVars.isHollow) {
            const thickness = genVars.thickness;
            const hLocal = [relLoc.x / (genVars.radiiOff[0] - thickness), relLoc.y / (genVars.radiiOff[1] - thickness), relLoc.z / (genVars.radiiOff[2] - thickness)];
            if (hLocal[0] * hLocal[0] + hLocal[1] * hLocal[1] + hLocal[2] * hLocal[2] < 1.0) {
                return false;
            }
        }
        const local = [relLoc.x / genVars.radiiOff[0], relLoc.y / genVars.radiiOff[1], relLoc.z / genVars.radiiOff[2]];
        if (local[0] * local[0] + local[1] * local[1] + local[2] * local[2] <= 1.0) {
            return true;
        }
        return false;
    }
}
